==> seplib-6.3.9.tar.gz: seplib-6.3.9/seplib_base/lib/corelibs/superset/sect_rite.c <==
#define SET_SDOC 1
#include<sitedef.h>
#include<superset.h>
#include<superset_internal.h>


_XFUNCPROTOBEGIN
int SEP3D_rite_section(sep_3d *info, sep_dff *info_dff, int ithread,  
 char *data, int write_data, int write_headers, int write_grid, int isect)
_XFUNCPROTOEND
{
int esize,nelem;
sep_3d *info_small;


info_dff->dff_rite=1; /*can't section names once written section*/

if(0!=sep3d_pass_headers(info_dff->impi,info->name,ithread,
   info_dff->sect_thread[isect])) return(sepwarn(NOT_MET,
   "tag=%s trouble passing section %d from %d to %d \n",
    info->name,isect,ithread,info_dff->sect_thread[isect]));

if(info_dff->impi == info_dff->sect_thread[isect] || info_dff->impi==ithread){
  if(write_data==1){
    esize=SEP3D_get_esize(info);
  
    nelem=info->ncoord*info->n[0];

    if(0!=sep3d_pass_data(info_dff->impi,ithread,
      info_dff->sect_thread[isect],nelem,esize,data))
      return(sepwarn(NOT_MET,"trouble passing data tag=%s \n",info->name));;
  }
}

if(info_dff->impi == info_dff->sect_thread[isect]){
  info_small= tag_info_section(info_dff,isect, INQUIRE);  
  if(info_small->ncoord>0){
    if(0!=SEP3D_rite(info_dff->tag_sect[isect],info,data,
      write_data,write_headers,write_grid))
      return(sepwarn(NOT_MET,"trouble writing to tag %s \n",
       info_dff->tag_sect[isect]));
  }
}
return(SUCCESS);
}
_XFUNCPROTOBEGIN
int SEP3D_rite_section_list(sep_3d *info, sep_dff *info_dff, 
  char *data, int write_data, int write_headers,
   int write_grid,int nsect, int *sect_list)
_XFUNCPROTOEND
{
int isect,n123,i,nh,idim;
int nout,fout,jout,nsz ;
int nt,ft,jt,*ntot,i1;
char *temp_data;
int *sect_index;
sep_3d *info_temp;
sep_3d *info_small;


 if(info->file_format==REGULAR){
   if(0!=SEP3D_wind_coords(info,&nsz))
     return(sepwarn(NOT_MET,
     "trouble grabbing coords from window parameters tag=%s \n",info->name));
       if(nsz==0) return(SUCCESS);
 }


    if(info->ncoord==0) return(SUCCESS);

    sect_index=(int*)malloc(sizeof(int)*info->ncoord);
    if(0!=SEP3D_find_sect_index(info,info,info_dff,sect_index)){
      free(sect_index);
       return(sepwarn(NOT_MET,
         "trouble finding sect index tag=%s \n",info->name));
    }



  ntot=(int*)malloc(sizeof(int)*info_dff->nsect);
  if(info_dff->sect_axis==1){
    for(i1=0; i1 < info_dff->nsect; i1++) ntot[i1]=info->ncoord;
  }else{
    for(i1=0; i1 < info_dff->nsect; i1++) ntot[i1]=0;
    for(i1=0; i1 < info->ncoord; i1++)  ntot[sect_index[i1]]++;
  }

  for(i=0; i < nsect; i++){
    isect=sect_list[i];
   
    info_temp= tag_info_section(info_dff,isect, INQUIRE);  
    if(info_temp==SEPNULL){
      info_temp= tag_info_section(info_dff,isect, SCRATCH);  
      if(info_temp==SEPNULL){
        free(sect_index); free(ntot);
        return(sepwarn(NOT_MET,"trouble making temporary structure tag=%s \n",
          info->name)); 
      }
    }

    if(0!=SEP3D_localize_window(info,info_temp,isect)){
      free(sect_index); free(ntot);
      return(sepwarn(NOT_MET,"trouble localizing window tag=%s sect=%d \n",
        info->name,isect));
    } 

    if(0!=SEP3D_alloc_coord(info_temp,ntot[isect])){
      free(sect_index); free(ntot);
      return(sepwarn(NOT_MET,"trouble allocating coordinates section=%d alloc=%d thread=%d\n",isect,ntot[isect],
       info_dff->impi));
    }

    if(info_temp->file_format!=REGULAR){
      if(0!=sep3d_set_nh(info_temp->name,ntot[isect]))
        sepwarn(NOT_MET,"trouble setting nh tag=%s\n",info_temp->name);
    }


    if(info_dff->sect_thread[isect]==info_dff->impi ){
     if(0!=SEP3D_grab_header_section(info,info_dff,info_temp,isect,info->ncoord,sect_index)){
       free(sect_index); free(ntot);
       return(sepwarn(NOT_MET,"tag=%s trouble grabbing header section \n",
        info->name));
     }
    }
   
   temp_data=(char*) malloc(sizeof(char*)*info_temp->n[0]*ntot[isect]*
    SEP3D_get_esize(info_temp));

    if(info_dff->sect_thread[isect]==info_dff->impi ){
     if(0!=SEP3D_grab_data_section(info,info_dff,info_temp,isect,
        data,temp_data,info->ncoord,sect_index)){
       free(sect_index); free(ntot);
       return(sepwarn(NOT_MET,"tag=%s trouble grabbing header section \n",
        info->name));
     }
    }
    if(0!=SEP3D_rite_section(info_temp,info_dff,info_dff->sect_thread[isect],
      temp_data, write_data, write_headers, write_grid,isect)){
        free(sect_index); free(ntot);
        return(sepwarn(NOT_MET,"tag=%s section=%d trouble writing seciton\n",
         info->name,isect));
    }

    free(temp_data);
    if(0!=sep3d_clear_headers(info_temp->name) &&
       0!=SEP3D_delete_coord(info_temp)){
          free(sect_index); free(ntot);
          return(sepwarn(NOT_MET,"tag=%s section=%d trouble writing seciton\n",
           info_temp->name,isect));
    }
  }

  free(sect_index); free(ntot);
  return(SUCCESS);
}

_XFUNCPROTOBEGIN
int sep3d_rite_local(char *tag, char *sep3dname, int *nwind, int *fwind,
  int *jwind,char *data, int ntraces, int write_data, int write_headers,
   int write_grid)
_XFUNCPROTOEND
{
sep_3d *info;
sep_dff *info_dff;
int  i,*sect_list,ierr;
char dff[512];


info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s:  sep3dname=%s invalid struc\n",tag,
    sep3dname));

if(info->dff == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  not a distributed tag \n",tag));


info_dff = info->dff;
if(info_dff == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  dff structure (%s) has not been setup \n",tag));

for(i=0; i < info->ndims; i++){
   info->nwind[i]=nwind[i];
   info->fwind[i]=fwind[i];
   info->jwind[i]=jwind[i];
}

  
                                                                                

ierr=SEP3D_rite_section_list(info,info_dff,data,write_data,
  write_headers,write_grid,info_dff->nsect_ithread,info_dff->sect_ithread);

return(ierr);
}

==> seplib-6.3.12.tar.gz: seplib-6.3.12/seplib_base/lib/corelibs/superset/sect_rite.c <==
#define SET_SDOC 1
#include<sitedef.h>
#include<superset.h>
#include<superset_internal.h>


_XFUNCPROTOBEGIN
int SEP3D_rite_section(sep_3d *info, sep_dff *info_dff, int ithread,  
 char *data, int write_data, int write_headers, int write_grid, int isect)
_XFUNCPROTOEND
{
int esize,nelem;
sep_3d *info_small;


info_dff->dff_rite=1; /*can't section names once written section*/

if(0!=sep3d_pass_headers(info_dff->impi,info->name,ithread,
   info_dff->sect_thread[isect],1)) return(sepwarn(NOT_MET,
   "tag=%s trouble passing section %d from %d to %d \n",
    info->name,isect,ithread,info_dff->sect_thread[isect]));

if(info_dff->impi == info_dff->sect_thread[isect] || info_dff->impi==ithread){
  if(write_data==1){
    esize=SEP3D_get_esize(info);
  
    nelem=info->ncoord*info->n[0];

    if(0!=sep3d_pass_data(info_dff->impi,ithread,
      info_dff->sect_thread[isect],nelem,esize,data))
      return(sepwarn(NOT_MET,"trouble passing data tag=%s \n",info->name));;
  }
}

if(info_dff->impi == info_dff->sect_thread[isect]){
  info_small= tag_info_section(info_dff,isect, INQUIRE);  
  if(info_small->ncoord>0){
    if(0!=SEP3D_rite(info_dff->tag_sect[isect],info,data,
      write_data,write_headers,write_grid))
      return(sepwarn(NOT_MET,"trouble writing to tag %s \n",
       info_dff->tag_sect[isect]));
  }
}
return(SUCCESS);
}
_XFUNCPROTOBEGIN
int SEP3D_rite_section_list(sep_3d *info, sep_dff *info_dff, 
  char *data, int write_data, int write_headers,
   int write_grid,int nsect, int *sect_list)
_XFUNCPROTOEND
{
int isect,n123,i,nh,idim;
int nout,fout,jout,nsz ;
int nt,ft,jt,*ntot,i1;
char *temp_data;
int *sect_index;
sep_3d *info_temp;
sep_3d *info_small;


 if(info->file_format==REGULAR){
   if(0!=SEP3D_wind_coords(info,&nsz))
     return(sepwarn(NOT_MET,
     "trouble grabbing coords from window parameters tag=%s \n",info->name));
       if(nsz==0) return(SUCCESS);
 }


    if(info->ncoord==0) return(SUCCESS);

    sect_index=(int*)malloc(sizeof(int)*info->ncoord);
    if(0!=SEP3D_find_sect_index(info,info,info_dff,sect_index)){
      free(sect_index);
       return(sepwarn(NOT_MET,
         "trouble finding sect index tag=%s \n",info->name));
    }



  ntot=(int*)malloc(sizeof(int)*info_dff->nsect);
  if(info_dff->sect_axis==1){
    for(i1=0; i1 < info_dff->nsect; i1++) ntot[i1]=info->ncoord;
  }else{
    for(i1=0; i1 < info_dff->nsect; i1++) ntot[i1]=0;
    for(i1=0; i1 < info->ncoord; i1++)  ntot[sect_index[i1]]++;
  }

  for(i=0; i < nsect; i++){
    isect=sect_list[i];
   
    info_temp= tag_info_section(info_dff,isect, INQUIRE);  
    if(info_temp==SEPNULL){
      info_temp= tag_info_section(info_dff,isect, SCRATCH);  
      if(info_temp==SEPNULL){
        free(sect_index); free(ntot);
        return(sepwarn(NOT_MET,"trouble making temporary structure tag=%s \n",
          info->name)); 
      }
    }

    if(0!=SEP3D_localize_window(info,info_temp,isect)){
      free(sect_index); free(ntot);
      return(sepwarn(NOT_MET,"trouble localizing window tag=%s sect=%d \n",
        info->name,isect));
    } 

    if(0!=SEP3D_alloc_coord(info_temp,ntot[isect])){
      free(sect_index); free(ntot);
      return(sepwarn(NOT_MET,"trouble allocating coordinates section=%d alloc=%d thread=%d\n",isect,ntot[isect],
       info_dff->impi));
    }

    if(info_temp->file_format!=REGULAR){
      if(0!=sep3d_set_nh(info_temp->name,ntot[isect]))
        sepwarn(NOT_MET,"trouble setting nh tag=%s\n",info_temp->name);
    }


    if(info_dff->sect_thread[isect]==info_dff->impi ){
     if(0!=SEP3D_grab_header_section(info,info_dff,info_temp,isect,info->ncoord,sect_index)){
       free(sect_index); free(ntot);
       return(sepwarn(NOT_MET,"tag=%s trouble grabbing header section \n",
        info->name));
     }
    }
   
   temp_data=(char*) malloc(sizeof(char*)*info_temp->n[0]*ntot[isect]*
    SEP3D_get_esize(info_temp));

    if(info_dff->sect_thread[isect]==info_dff->impi ){
     if(0!=SEP3D_grab_data_section(info,info_dff,info_temp,isect,
        data,temp_data,info->ncoord,sect_index)){
       free(sect_index); free(ntot);
       return(sepwarn(NOT_MET,"tag=%s trouble grabbing header section \n",
        info->name));
     }
    }
    if(0!=SEP3D_rite_section(info_temp,info_dff,info_dff->sect_thread[isect],
      temp_data, write_data, write_headers, write_grid,isect)){
        free(sect_index); free(ntot);
        return(sepwarn(NOT_MET,"tag=%s section=%d trouble writing seciton\n",
         info->name,isect));
    }

    free(temp_data);
    if(0!=sep3d_clear_headers(info_temp->name) &&
       0!=SEP3D_delete_coord(info_temp)){
          free(sect_index); free(ntot);
          return(sepwarn(NOT_MET,"tag=%s section=%d trouble writing seciton\n",
           info_temp->name,isect));
    }
  }

  free(sect_index); free(ntot);
  return(SUCCESS);
}

_XFUNCPROTOBEGIN
int sep3d_rite_local(char *tag, char *sep3dname, int *nwind, int *fwind,
  int *jwind,char *data, int ntraces, int write_data, int write_headers,
   int write_grid)
_XFUNCPROTOEND
{
sep_3d *info;
sep_dff *info_dff;
int  i,*sect_list,ierr;
char dff[512];


info = tag_info_sep3d(sep3dname, INQUIRE);  /* get info on this tag */
if(info == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s:  sep3dname=%s invalid struc\n",tag,
    sep3dname));

if(info->dff == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  not a distributed tag \n",tag));


info_dff = info->dff;
if(info_dff == SEPNULL)
  return (sepwarn(INVALID_STRUC,"tag:%s  dff structure (%s) has not been setup \n",tag));

for(i=0; i < info->ndims; i++){
   info->nwind[i]=nwind[i];
   info->fwind[i]=fwind[i];
   info->jwind[i]=jwind[i];
}

  
                                                                                

ierr=SEP3D_rite_section_list(info,info_dff,data,write_data,
  write_headers,write_grid,info_dff->nsect_ithread,info_dff->sect_ithread);

return(ierr);
}
